@@ -1,3 +1,4 @@
+# coding=utf8
 #  Copyright 2021- imbus AG
 #
 #  Licensed under the Apache License, Version 2.0 (the "License");
@@ -12,7 +13,7 @@
 #  See the License for the specific language governing permissions and
 #  limitations under the License.
 
-from __future__ import annotations
+# from __future__ import annotations
 
 import base64
 from typing import Optional, Union, List, Dict
@@ -20,14 +21,15 @@ import requests
 import urllib3
 import time
 from TestBenchCliReporter.actions import AbstractAction
-from TestBenchCliReporter.util import XmlExportConfig, ImportConfig
+from TestBenchCliReporter import questions
+from TestBenchCliReporter.util import XmlExportConfig, ImportConfig, close_program
 from questionary import print as pprint
 import json
 import os
 
 
 def spinner():
-    if os.name == "nt":
+    if os.name != "posix":
         return ["_", "_", "_", "-", "`", "`", "'", "´", "-", "_", "_", "_"]
     else:
         return [
@@ -99,43 +101,15 @@ def delay():
 
 
 def spin_spinner(message: str):
-    for cursor in spinner():
-        print(
-            f"{message} {cursor}",
-            end="\r",
-        )
-        time.sleep(delay())
-
-
-class ConnectionLog:
-    def __init__(self):
-        self.connections: list[Connection] = []
-
-    @property
-    def len(self) -> int:
-        return len(self.connections)
-
-    @property
-    def active_connection(self) -> Connection:
-        return self.connections[-1]
-
-    def next(self):
-        self.connections = self.connections[1:] + self.connections[:1]
-
-    def remove(self, connection):
-        self.connections.remove(connection)
-
-    def add_connection(self, new_connection: Connection):
-        self.connections.append(new_connection)
-
-    def export_as_json(self, output_file_path: str):
-        print("Generating JSON export")
-        export_dict = {
-            "configuration": [connection.export() for connection in self.connections]
-        }
-
-        with open(output_file_path, "w") as output_file:
-            json.dump(export_dict, output_file, indent=2)
+    try:
+        for cursor in spinner():
+            print(
+                f"{message} {cursor}",
+                end="\r",
+            )
+            time.sleep(delay())
+    except UnicodeEncodeError:
+        pass
 
 
 class Connection:
@@ -161,7 +135,7 @@ class Connection:
             self.password = password
         self.job_timeout_sec = job_timeout_sec
         self.action_log: List[AbstractAction] = []
-        self.actions_to_trigger: List[Dict] = actions or []
+        self.actions_to_trigger: List[dict] = actions or []
         self.actions_to_wait_for: List[AbstractAction] = []
         self.actions_to_finish: List[AbstractAction] = []
         self.connection_timeout = connection_timeout_sec
@@ -204,7 +178,7 @@ class Connection:
     def add_action(self, action: AbstractAction):
         self.action_log.append(action)
 
-    def check_is_identical(self, other: Connection) -> bool:
+    def check_is_identical(self, other) -> bool:
         if (
             self.server_url == other.server_url
             and self.loginname == other.loginname
@@ -227,7 +201,7 @@ class Connection:
 
         return True
 
-    def get_all_projects(self) -> dict:
+    def get_all_projects(self) -> Dict:
         all_projects = self.session.get(
             self.server_url + "projects",
             params={"includeTOVs": "true", "includeCycles": "true"},
@@ -235,7 +209,7 @@ class Connection:
         all_projects["projects"].sort(key=lambda proj: proj["name"].casefold())
         return all_projects
 
-    def get_all_filters(self) -> list[dict]:
+    def get_all_filters(self) -> List[dict]:
         all_filters = self.session.get(
             self.server_url + "filters",
         )
@@ -272,7 +246,7 @@ class Connection:
             report_config["reportRootUID"] = reportRootUID
         report_config["filters"]: filters
 
-        if cycle_key:
+        if cycle_key and cycle_key != "0":
             response = self.session.post(
                 self.server_url + "cycle/" + cycle_key + "/xmlReport",
                 json=report_config,
@@ -316,14 +290,14 @@ class Connection:
 
         return report.content
 
-    def get_all_testers_of_project(self, project_key: str) -> list[dict]:
+    def get_all_testers_of_project(self, project_key: str) -> List[dict]:
         return [
             member
             for member in self.get_all_members_of_project(project_key)
             if "Tester" in member["value"]["membership"]["roles"]
         ]
 
-    def get_all_members_of_project(self, project_key: str) -> list[dict]:
+    def get_all_members_of_project(self, project_key: str) -> List[dict]:
         all_project_members = self.session.get(
             self.server_url + "project/" + project_key + "/members",
         )
@@ -348,7 +322,7 @@ class Connection:
         report_root_uid: str,
         serverside_file_name: str,
         default_tester: str,
-        filters: list[dict[str, str]],
+        filters: List[Dict[str, str]],
         import_config: Dict = ImportConfig["Typical"],
     ) -> str:
 
@@ -409,19 +383,72 @@ class Connection:
         else:
             raise AssertionError(result)
 
-    def get_test_cycle_structure(self, cycle_key: str) -> list[dict]:
+    def get_test_cycle_structure(self, cycle_key: str) -> List[dict]:
         test_cycle_structure = self.session.get(
             self.server_url + "cycle/" + cycle_key + "/structure",
         )
         return test_cycle_structure.json()
 
-    def get_tov_structure(self, tovKey: str) -> list[dict]:
+    def get_tov_structure(self, tovKey: str) -> List[dict]:
         tov_structure = self.session.get(
             self.server_url + "tov/" + tovKey + "/structure",
         )
         return tov_structure.json()
 
 
+def login(server="", login="", pwd="") -> Connection:
+    if server and login and pwd:
+        credentials = {
+            "server_url": server,
+            "verify": False,
+            "loginname": login,
+            "password": pwd,
+        }
+    else:
+        credentials = questions.ask_for_test_bench_credentials(server, login, pwd)
+
+    while True:
+        connection = Connection(**credentials)
+        try:
+            if connection.check_is_working():
+                return connection
+
+        except requests.HTTPError:
+            print("Invalid login credentials.")
+            action = questions.ask_for_action_after_failed_login()
+            if action == "retry_password":
+                credentials["password"] = questions.ask_for_testbench_password()
+            elif action == "change_user":
+                credentials["loginname"] = questions.ask_for_testbench_loginname()
+                credentials["password"] = questions.ask_for_testbench_password()
+            elif action == "change_server":
+                credentials = questions.ask_for_test_bench_credentials()
+            else:
+                close_program()
+
+        except (requests.ConnectionError, requests.exceptions.MissingSchema):
+            print("Invalid server url.")
+            action = questions.ask_for_action_after_failed_server_connection()
+            if action == "retry_server":
+                credentials["server_url"] = questions.ask_for_test_bench_server_url()
+            elif action == "change_server":
+                credentials = questions.ask_for_test_bench_credentials()
+            else:
+                close_program()
+
+        except requests.exceptions.Timeout:
+            print("No connection could be established due to timeout.")
+            action = questions.ask_for_action_after_login_timeout()
+            if action == "retry":
+                pass
+            elif action == "retry_server":
+                credentials["server_url"] = questions.ask_for_test_bench_server_url()
+            elif action == "change_server":
+                credentials = questions.ask_for_test_bench_credentials()
+            else:
+                close_program()
+
+
 class TimeoutHTTPAdapter(requests.adapters.HTTPAdapter):
     def __init__(self, timeout: Optional[int] = 60, *args, **kwargs):
         self.timeout = timeout
@@ -434,3 +461,34 @@ class TimeoutHTTPAdapter(requests.adapters.HTTPAdapter):
 
 class JobTimeout(requests.exceptions.Timeout):
     pass
+
+
+class ConnectionLog:
+    def __init__(self):
+        self.connections: List[Connection] = []
+
+    @property
+    def len(self) -> int:
+        return len(self.connections)
+
+    @property
+    def active_connection(self) -> Connection:
+        return self.connections[-1]
+
+    def next(self):
+        self.connections = self.connections[1:] + self.connections[:1]
+
+    def remove(self, connection):
+        self.connections.remove(connection)
+
+    def add_connection(self, new_connection: Connection):
+        self.connections.append(new_connection)
+
+    def export_as_json(self, output_file_path: str):
+        print("Generating JSON export")
+        export_dict = {
+            "configuration": [connection.export() for connection in self.connections]
+        }
+
+        with open(output_file_path, "w") as output_file:
+            json.dump(export_dict, output_file, indent=2)
